class UpdateScoreFromRequestJob < ActiveJob::Base
  queue_as :x_hermes_score_requests

  def perform(applicant_id, is_new_request)
    i=0
    applicant = Hermes::Applicant.find applicant_id
    offers = Hermes::HousingOffer.only(:id, :effort_rate, :lessor_id, :typology).where(
      **applicant.eligibility_query.json_to_h.merge(offer_status: {:$in => Hermes::HousingOffer.published_statuses})
    )
    total = offers.length
    
    return if offers.blank?

    found_or_created_scores = []
    offers.each do |offer|
      next unless offer && applicant

      UpdateOfferEligibleNbJob.perform_later(offer.id.to_s)

      score = Hermes::Score.find_or_create_by(housing_offer_id: offer.id, applicant_id: applicant_id, is_active: true) do
        offer.notify(key: NOTIFICATION_KEYS[:housing_offer][:new_eligible_offer], channels: ['email'], sender: offer.lessor)
      end
      score.process

      Rails.logger.debug "calc score for offer [#{i}/#{total}] #{offer.id} applicant : #{applicant_id} score : #{score.id}"
      found_or_created_scores << offer.id.to_s
    end

    offer_filter = { :$nin => found_or_created_scores }
    applicant_filter = applicant_id

    other_scores = Hermes::Score.where(housing_offer_id: offer_filter, applicant_id: applicant_filter)
    other_scores.set(is_active: false, updated_at: DateTime.current)

    # This lines was at the end of old UpdateScoreJob file
    # they was initialized only for housing_request types
    # but offers_true_filter hash wasn't be really used
    # i think they was used on old logic (david)

    # offers_true_filter = offer_filter
    # offers_true_filter[:$in] = offers_true_filter[:$nin]
    # offers_true_filter.delete(:$nin)

    Rails.logger.debug "#{other_scores.count} scores have been set to inactive for housing_request : #{applicant_id}"
  end
end
