# frozen_string_literal: true
=begin


# exemple d'utilisation
=begin

  hr = Hermes::HousingRequest.find("5e3d823ca70bf25ace59e9eb")

 so=Hermes::BoClient::AddRequestTask.new({housing_request_id: hr.id})
 so.to_xml


ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml)
=end

module Hermes
  module BoClient
    class AddRequestTask < BaseTask
      
      def initialize(options = {})
        @xsd = "InterfaceNuu_v0300.xsd"
        super(options)
      end
      
      endpoint {'/ws/rest/als/demande/v1/integrer'}
      
      task_name 'addrequest'
      type_flux 'BIEN'
      mapping 'Hermes::BoClient::Mappings::HousingRequestMap'
      
      params *%i[
                numGuichet
                refInterne
                dateCreationDemande
                forcerDoublon
                renouvellementElec
                categorieLogementRecherche
                rezDeChaussee
                sansAscenseur
                montantLoyer
                demandeElargie
                typeLogement
                commune
                codePostal
                motifDemande
                civilite
                nom
                prenom
                dateNaissance
                nationalite
                situationFamiliale
                etranger
                batiment
                numero
                voie
                codePostal
                commune
                typeContratTravail
                multiEmployeur
                anneeMoins1
                montantMoins1
                etranger
                numero
                voie
                codePostal
                commune
                modeleLogement
                montantMensuelLoyer
                montantMensuelAPL
                nombreHabitant
                surface
                proprietaire
                etranger
            }
      ]
      
      def add_if_exists(b, item_name, mapping_name = nil, is_code = false)
        mapping_name ||= item_name
        mapping_value = mapping.send(mapping_name)
        
        b.send(item_name, is_code ? {"code" => mapping_value} : mapping_value) unless mapping_value == nil
      
      end
      
      def to_xml
        namespaces = {
          'xmlns' => 'http://nuu.application.i2/',
          'xmlns:xsi' => "http://www.w3.org/2001/XMLSchema-instance",
          'versionInterface' => "03.00"
        }
        builder = Nokogiri::XML::Builder.new(encoding: 'UTF-8') do |b|
          b.interfaceNuu(namespaces) do
            b.parent.namespace = nil
            b.entete do
              b.typeFichier "CRE" #TODO : A modifier une fois que la gestion de doc aura ete implementée
              b.dateFichier DateTime.current
            end
            b.demande do
              b.demandeLogement do
                add_if_exists b, "numGuichet"
                add_if_exists b, "numUnique"
                
                add_if_exists b, "refInterne"
                add_if_exists b, "dateCreationDemande"
                add_if_exists b, "forcerDoublon"
                add_if_exists b, "renouvellementElec"
                add_if_exists b, "precision"
                add_if_exists b, "ressourcesLoiEC"
                b.logementRecherche do
                  add_if_exists b, "categorieLogementRecherche", "categorieLogementRecherche", true
                  add_if_exists b, "avecParking"
                  add_if_exists b, "rezDeChaussee"
                  
                  add_if_exists b, "sansAscenseur"
                  add_if_exists b, "montantLoyer"
                  add_if_exists b, "demandeElargie"
                  
                  b.listeTypeLogement do
                    mapping.typeLogement&.each do |type|
                      b.typeLogement({"code" => type})
                    end
                  end
                  
                  #todo : peut etre qu'il faut revoir ici ce qu'il y a a faire
                  b.listeLocalisationSouhaite do
                    mapping.localisationSouhaite&.each_with_index do |value, i|
                      b.localisationSouhaite do
                        b.commune({"code" => value[:code]})
                        b.codePostal (value[:codePostal])
                        b.seuilEPCIPremierQuartile (value[:seeked_epci_first_quartil]) if (value[:seeked_epci_first_quartil])
                        b.quartileAppartenance (value[:seeked_quartil_belongings])
                      
                      end
                    end
                  end
                end
                b.principauxMotifDemande do
                  mapping.motifDemande&.each do |motif|
                    b.motifDemande({"code" => motif})
                  end
                
                end
                b.personnePhysique do
                  b.demandeur do
                    add_if_exists b, "civilite", "civilite", true
                    add_if_exists b, "nom"
                    add_if_exists b, "nomPatronymique"
                    add_if_exists b, "prenom"
                    add_if_exists b, "nir"
                    add_if_exists b, "dateNaissance"
                    add_if_exists b, "nationalite", "nationalite", true
                    add_if_exists b, "situationFamiliale", "situationFamiliale", true
                    add_if_exists b, "telDomicile"
                    add_if_exists b, "telPortable"
                    add_if_exists b, "telTravail"
                    add_if_exists b, "mel"
                    add_if_exists b, "notifSms"
                    add_if_exists b, "notifEvt"
                    add_if_exists b, "melAssistantDemarches"
                    b.adressePostale do
                      add_if_exists b, "etranger", "etrangerDem"
                      add_if_exists b, "appartement", "appartementDem"
                      add_if_exists b, "etage", "etageDem"
                      add_if_exists b, "escalier", "escalierDem"
                      add_if_exists b, "batiment", "batimentDem"
                      add_if_exists b, "compAdresse", "compAdresseDem"
                      add_if_exists b, "numero", "numeroDem"
                      add_if_exists b, "voie", "voieDem"
                      add_if_exists b, "lieuDit", "lieuDitDem"
                      add_if_exists b, "codePostal", "codePostalDem"
                      add_if_exists b, "codePostalEtranger", "codePostalEtrangerDem"
                      add_if_exists b, "communeEtranger", "communeEtrangerDem"
                      add_if_exists b, "pays", "paysDem"
                      add_if_exists b, "commune", "communeDem", true
                      add_if_exists b, "nomHebergeant", "nomHebergeantDem"
                    end
                    b.situationProfessionnelle do
                      add_if_exists b, "profession"
                      add_if_exists b, "typeContratTravail", "typeContratTravail", true
                      add_if_exists b, "multiEmployeur"
                      add_if_exists b, "nomEmployeurPrincipal"
                      add_if_exists b, "etranger", "etrangerProf"
                      add_if_exists b, "codePostal", "codePostalProf"
                      add_if_exists b, "codePostalEtranger", "codePostalEtrangerProf"
                      add_if_exists b, "communeEtranger", "communeEtrangerProf"
                      add_if_exists b, "pays", "paysProf"

                      add_if_exists(b, "commune", "communeProf", true) if (mapping.communeProf && mapping.communeProf != "")
                      add_if_exists b, "siretEmployeur"
                    end
                    
                    b.revenuFiscal do
                      if (mapping.anneeMoins1 || mapping.anneeMoins2)
                        add_if_exists b, "anneeMoins1"
                        add_if_exists b, "montantMoins1"
                        add_if_exists b, "anneeMoins2"
                        add_if_exists b, "montantMoins2"
                      end
                    end
                    b.listeRessourceRecue do
                      mapping.listeRessourceRecue&.each do |key, val|
                        b.detailRessource do
                          b.ressource({"code" => key})
                          b.montant val
                        end
                      end
                    end
                    b.listeRessourceVersee do
                      mapping.listeRessourceVersee&.each do |key, val|
                        b.detailRessource do
                          b.ressource({"code" => key})
                          b.montant val
                        end
                      end
                    end
                    b.listeRessourceRecuePersACharge do
                      mapping.listeRessourceRecuePersACharge&.each do |pers|
                        next unless pers.respond_to?(:each)
                        (incomes = pers&.fetch('monthly_incomes', nil)) &&
                          incomes.respond_to?(:each) &&
                          incomes&.each do |key, val|
                            b.detailRessource do
                              b.ressource({"code" => key})
                              b.montant val
                            end
                          end
                      end
                    end
                    b.listeRessourceVerseePersACharge do
                      mapping.listeRessourceVerseePersACharge&.each do |pers|
                        next unless pers.respond_to?(:each)
                        (charges = pers&.fetch('monthly_charges', nil)) &&
                          charges.respond_to?(:each) &&
                          charges&.each do |key, val|
                            b.detailRessource do
                              b.ressource({"code" => key})
                              b.montant val
                            end
                          end
                      end
                    end
                  end
                  if (mapping.appartementLog || mapping.etageLog || mapping.escalierLog || mapping.batimentLog || mapping.compAdresseLog || mapping.numeroLog || mapping.voieLog)
                    b.adresseLogement do
                      add_if_exists b, "etranger", "etrangerLog"
                      add_if_exists b, "appartement", "appartementLog"
                      add_if_exists b, "etage", "etageLog"
                      add_if_exists b, "escalier", "escalierLog"
                      add_if_exists b, "batiment", "batimentLog"
                      add_if_exists b, "compAdresse", "compAdresseLog"
                      add_if_exists b, "numero", "numeroLog"
                      add_if_exists b, "voie", "voieLog"
                      add_if_exists b, "lieuDit", "lieuDitLog"
                      add_if_exists b, "codePostal", "codePostalLog"
                      add_if_exists b, "codePostalEtranger", "codePostalEtrangerLog"
                      add_if_exists b, "communeEtranger", "communeEtrangerLog"
                      add_if_exists b, "pays", "paysLog"
                      add_if_exists b, "commune", "communeLog", true
                      add_if_exists b, "nomHebergeant", "nomHebergeantLog"
                    end
                  end
                  b.listePersonneACharge do
                    
                    mapping.listePersonneACharge&.each do |pers|
                      if pers["rel_type"]
                        b.personneACharge do
                          b.nom pers["last_name"]
                          b.prenom pers["first_name"]
                          b.dateNaissance pers["birth_date"]&.to_date
                          b.sexe ({"code" => pers["sex"]})
                          b.lienParente ({"code" => pers["rel_type"]})
                        end
                      end
                    end
                  end
                  if mapping.personneANaitre && mapping.personneANaitre[0]
                    b.personneANaitre do
                      b.nombre (mapping.personneANaitre[0]["nb_child_birth"])
                      b.dateNaissancePrevue (mapping.personneANaitre[0]["anticipated_birth"]&.to_date)
                    end
                  end
                  b.listePersonneEnGarde do
                    
                    mapping.listePersonneACharge&.each do |pers|
                      if pers["coparentality"]
                        b.personneEnGarde do
                          b.nom pers["last_name"]
                          b.prenom pers["first_name"]
                          b.dateNaissance pers["birth_date"]&.to_date
                          b.sexe ({"code" => pers["sex"]})
                          b.coParentalite ({"code" => pers["coparentality"]})
                        end
                      end
                    
                    end
                  end
                  
                  
                  if (mapping.coapplicants && mapping.coapplicants.length && mapping.coapplicants != [nil])
                    b.listeCotitulaire do
                      
                      mapping.coapplicants.each_with_index do |coapp, i|
                        if coapp
                          b.cotitulaire do
                            applicant = Hermes::Applicant.find(coapp[:id])
                            b.civilite ({"code" => applicant.civility})
                            b.nom (applicant.last_name)
                            b.nomPatronymique (applicant.patronymic_name)
                            b.prenom (applicant.first_name)
                            b.nir (applicant.ss_id)
                            b.dateNaissance (applicant.birth_date&.to_date)
                            b.nationalite ({"code" => applicant.nationality})
                            b.situationFamiliale ({"code" => applicant.family_situation})
                            b.telDomicile (applicant.homephone) if applicant.homephone
                            b.telPortable (applicant.cellphone) if applicant.cellphone
                            b.telTravail (applicant.workphone) if applicant.workphone
                            b.mel (applicant.email)
                            b.lienDemandeur ({"code" => mapping.lienDemandeur(i)})
                            
                            b.identifiantTemporaire (applicant.temporary_id) if applicant.temporary_id
                            b.situationProfessionnelle do
                              b.profession (applicant&.profession_details&.fetch("profession", nil))
                              b.typeContratTravail ({"code" => applicant.work_contract_type})
                              b.multiEmployeur (applicant&.has_other_employers ? 1 : 0)
                              b.nomEmployeurPrincipal (applicant&.main_employer_name) if applicant&.main_employer_name
                              b.etranger ((applicant&.profession_details&.fetch("foreign", nil) && applicant&.profession_details&.fetch("foreign", nil) != "false") ? 1 : 0)
                              b.codePostal (applicant&.workplace_postal_code)
                                b.commune({"code" => (applicant&.workplace_insee_code)})
                           
                              add_if_exists b, "codePostalEtranger", "codePostalEtrangerProf"
                              add_if_exists b, "communeEtranger", "communeEtrangerProf"
                              add_if_exists b, "pays", "paysProf"
                              b.codePostalEtranger (applicant.foreign_workplace_postal_code) if applicant.foreign_workplace_postal_code
                              b.communeEtranger (applicant.foreign_workplace_locality) if applicant.foreign_workplace_locality
                              b.pays (applicant.foreign_workplace_country) if applicant.foreign_workplace_country
                              b.siretEmployeur (applicant.main_employer_siret) if applicant.main_employer_siret
                            
                            end
                            b.revenuFiscal do
                              b.anneeMoins1 (applicant.year_minus_one) if applicant.year_minus_one
                              b.montantMoins1 (applicant.amount_year_minus_one&.round.to_s) if applicant.amount_year_minus_one
                              b.anneeMoins2 (applicant.year_minus_two) if applicant.year_minus_two
                              b.montantMoins2 (applicant.amount_year_minus_two&.round.to_s) if applicant.amount_year_minus_two
                            end
                            b.listeRessourceRecue do
                              applicant&.monthly_incomes&.each do |key, val|
                                
                                b.detailRessource do
                                  b.ressource({"code" => key})
                                  b.montant (val)
                                end
                              end
                            end
                            b.listeRessourceVersee do
                              applicant&.monthly_charges&.each do |key, val|
                                
                                b.detailRessource do
                                  b.ressource({"code" => key})
                                  b.montant (val)
                                end
                              end
                            end
                          end
                        end
                      end
                    end
                  end
                  b.logementActuel do
                    b.listeModeleLogement do
                      if mapping.listeModeleLogement && mapping.listeModeleLogement.length > 1
                        mapping.listeModeleLogement.each do |val|
                          b.modeleLogement ({"code" => val})
                        end
                      else
                        b.modeleLogement  ({"code" => mapping.listeModeleLogement[0]})
                      end
                    
                    end
                    add_if_exists b, "compNomOrgBailleur"
                    add_if_exists b, "compSIRENOrgBailleur"
                    add_if_exists b, "montantMensuelLoyer"
                    add_if_exists b, "montantMensuelAPL"
                    add_if_exists b, "nombreHabitant"
                    add_if_exists b, "categorieLogement", "categorieLogementAct", true
                    add_if_exists b, "typeLogementActuel", "typeLogementActuelAct", true
                    add_if_exists b, "surface"
                    add_if_exists b, "proprietaire"
                    add_if_exists b, "etranger", "etrangerLogAct"
                    add_if_exists b, "codePostal", "codePostalLogAct"
                    add_if_exists b, "commune", "communeLogAct", true
                    add_if_exists b, "typeLogementPossede", "typeLogementPossede", true
                  end
                end
                if mapping.listeHandicap
                  b.listeHandicap do
                    mapping.listeHandicap.each do |handicap|
                      b.handicap do
                        b.dateNaissance (handicap["birth_date"]&.to_date)
                        if referent = handicap["disability_referent"]
                          b.referent do
                            b.handicapReconnu (referent["recognized_disability"] === true ? 1 : 0)
                            b.procheDomicile (referent["near_home"] === true ? 1 : 0)
                            b.nom (referent["name"])
                            b.adresse (referent["address"])
                            b.tel (referent["phone"])
                            b.mel (referent["email"])
                          end
                        end
                        b.compAutreNature (handicap["special_need"])
                        b.etatHandicap({"code" => handicap["disability_states"]})
                        b.capaciteMarcheHandicap({"code" => handicap["walking_capacities"]})
                        b.accompagne (handicap["accompanied"] === true ? 1 : 0)
                        b.handicapMoteur (handicap["physical_disability"] === true ? 1 : 0)
                        b.handicapSensoriel (handicap["sensory_disability"] === true ? 1 : 0)
                        b.handicapAutre (handicap["other_disability"] === true ? 1 : 0)
                        
                        b.listeNatureHandicapSensoriel do
                          handicap["sensory_disability_kinds"]&.each do |val|
                            b.natureHandicapSensoriel ({"code" => val})
                          end
                        end
                        b.listeNatureHandicapMoteur do
                          handicap["physical_disability_kinds"]&.each do |val|
                            b.natureHandicapMoteur ({"code" => val})
                          end
                        end
                        b.listeAideTechniqueHandicap do
                          handicap["disability_technical_helps"]&.each do |val|
                            b.aideTechniqueHandicap ({"code" => val})
                          end
                        end
                        b.listeEquipementHandicap do
                          handicap["disability_equipments"]&.each do |val|
                            b.equipementHandicap ({"code" => val})
                          end
                        end
                        b.besoinAutre (handicap["other_need"])
                      end
                    end
                  end
                end
                if mapping.dateRadiation && mapping.motifRadiation
                  b.radiation do
                    add_if_exists b, "dateRadiation"
                    add_if_exists b, "motifRadiation", "motifRadiation", true
                  end
                end
              
              
              end
            end
          end
        end
        
        builder.to_xml
      end
    end
  end
end
